@@ -1,9 +1,10 @@
 import copy
 import re
+import warnings
 
 import numpy as np
-import pint
 import pandas as pd
+import pint
 from pandas import DataFrame, Series
 from pandas.api.extensions import (
     ExtensionArray,
@@ -12,36 +13,15 @@ from pandas.api.extensions import (
     register_extension_dtype,
     register_series_accessor,
 )
-from pandas.api.types import is_integer, is_list_like, is_scalar
+from pandas.api.types import is_integer, is_list_like, is_object_dtype, is_string_dtype
 from pandas.compat import set_function_name
-from pandas.core import ops
-from pandas.arrays import BooleanArray, IntegerArray
 from pandas.core.arrays.base import ExtensionOpsMixin
-from pint import errors, compat
+from pandas.core.indexers import check_array_indexer
+from pint import compat, errors
 from pint.quantity import _Quantity
 from pint.unit import _Unit
 
 
-def convert_indexing_key(key):
-    if isinstance(key, BooleanArray):
-        return key.to_numpy(dtype=np.bool, na_value=False)
-
-    elif isinstance(key, IntegerArray):
-        if pd.isna(key).any():
-            raise ValueError("Cannot index with an integer indexer containing NA values")
-        return key.to_numpy(dtype=np.int)
-
-    elif isinstance(key, (list, tuple)):
-        if all(isinstance(val, bool) for val in key if val is not pd.NA):
-            return np.asarray([(False if val is pd.NA else val) for val in key], dtype=np.bool)
-        if all(isinstance(val, int) for val in key if val is not pd.NA):
-            if any(val is pd.NA for val in key):
-                raise ValueError("Cannot index with an integer indexer containing NA values")
-            return np.asarray([val for val in key if val is not pd.NA], dtype=np.int)
-
-    return key
-
-
 class PintType(ExtensionDtype):
     """
     A Pint duck-typed class, suitable for holding a quantity (with unit specified) dtype.
@@ -55,7 +35,7 @@ class PintType(ExtensionDtype):
     _metadata = ("units",)
     _match = re.compile(r"(P|p)int\[(?P<units>.+)\]")
     _cache = {}
-    ureg = pint.UnitRegistry()
+    ureg = pint.get_application_registry()
 
     @property
     def _is_numeric(self):
@@ -74,8 +54,6 @@ class PintType(ExtensionDtype):
 
         elif units is None:
             # empty constructor for pickle compat
-            # import pdb
-            # pdb.set_trace()
             return object.__new__(cls)
 
         if not isinstance(units, _Unit):
@@ -114,6 +92,10 @@ class PintType(ExtensionDtype):
         Strict construction from a string, raise a TypeError if not
         possible
         """
+        if not isinstance(string, str):
+            raise TypeError(
+                f"'construct_from_string' expects a string, got {type(string)}"
+            )
         if isinstance(string, str) and (
             string.startswith("pint[") or string.startswith("Pint[")
         ):
@@ -123,14 +105,21 @@ class PintType(ExtensionDtype):
                 return cls(units=string)
             except ValueError:
                 pass
-        # This else block may allow pd.Series([1,2],dtype="m")
-        #         else:
-        #             try:
-        #                 return cls(units=string)
-        #             except ValueError:
-        #                 pass
+        raise TypeError(f"Cannot construct a 'PintType' from '{string}'")
 
-        raise TypeError("Cannot construct a 'PintType' from '{}'".format(string))
+    @classmethod
+    def construct_from_quantity_string(cls, string):
+        """
+        Strict construction from a string, raise a TypeError if not
+        possible
+        """
+        if not isinstance(string, str):
+            raise TypeError(
+                f"'construct_from_quantity_string' expects a string, got {type(string)}"
+            )
+
+        quantity = cls.ureg.Quantity(string)
+        return cls(units=quantity.units)
 
     # def __unicode__(self):
     # return compat.text_type(self.name)
@@ -141,7 +130,7 @@ class PintType(ExtensionDtype):
 
     @property
     def na_value(self):
-        return np.nan * self.units
+        return self.ureg.Quantity(np.nan, self.units)
 
     def __hash__(self):
         # make myself hashable
@@ -193,18 +182,21 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
     context_name = None
     context_units = None
 
-    def __init__(self, values, dtype=None, copy=False, data_dtype=None):
+    def __init__(self, values, dtype=None, copy=False):
         if dtype is None:
             raise NotImplementedError
 
         if not isinstance(dtype, PintType):
             dtype = PintType(dtype)
         self._dtype = dtype
-        if len(values) == 0:
-            data_dtype = "float"
-        else:
-            data_dtype = type(values[0])
-        self._data = np.array(values, data_dtype, copy=copy)
+        if len(values) > 0 and all([not isinstance(x, float) for x in values]):
+            data_dtype = next(x for x in values if not isinstance(x, float))
+            warnings.warn(
+                f"pint-pandas does not support magnitudes of {type(data_dtype)}. Converting magnitudes to float.",
+                category=RuntimeWarning,
+            )
+        self._data = np.array(values, float, copy=copy)
+        self._Q = self.dtype.ureg.Quantity
 
     @property
     def dtype(self):
@@ -239,12 +231,11 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
         if is_integer(item):
             return self._data[item] * self.units
 
-        item = convert_indexing_key(item)
+        item = check_array_indexer(self, item)
 
         return self.__class__(self._data[item], self.dtype)
 
     def __setitem__(self, key, value):
-
         # need to not use `not value` on numpy arrays
         if isinstance(value, (list, tuple)) and (not value):
             # doing nothing here seems to be ok
@@ -254,15 +245,13 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
             value = value.to(self.units).magnitude
         elif is_list_like(value) and isinstance(value[0], _Quantity):
             value = [item.to(self.units).magnitude for item in value]
-        _is_scalar = is_scalar(value)
-        if _is_scalar:
-            value = [value]
 
-        if _is_scalar:
-            value = value[0]
-
-        key = convert_indexing_key(key)
-        self._data[key] = value
+        key = check_array_indexer(self, key)
+        try:
+            self._data[key] = value
+        except IndexError as e:
+            msg = "Mask is wrong length. {}".format(e)
+            raise IndexError(msg)
 
     def _formatter(self, boxed=False):
         """Formatting function for scalar values.
@@ -331,10 +320,10 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
         ):
             dtype = PintType(dtype)
         if isinstance(dtype, PintType):
-            if dtype == self._dtype:
+            if dtype == self._dtype and not copy:
                 return self
             else:
-                return PintArray(self.quantity.to(dtype.units), dtype)
+                return PintArray(self.quantity.to(dtype.units).magnitude, dtype)
         return self.__array__(dtype, copy)
 
     @property
@@ -343,7 +332,7 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
 
     @property
     def quantity(self):
-        return self.data * self._dtype.units
+        return self._Q(self.data, self._dtype.units)
 
     def take(self, indices, allow_fill=False, fill_value=None):
         """Take elements from an array.
@@ -454,10 +443,14 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
         if isinstance(master_scalar, _Quantity):
             scalars = [quantify_nan(item) for item in scalars]
             scalars = [item.to(dtype.units).magnitude for item in scalars]
-        # import pdb
-        # pdb.set_trace()
         return cls(scalars, dtype=dtype, copy=copy)
 
+    @classmethod
+    def _from_sequence_of_strings(cls, scalars, dtype=None, copy=False):
+        if not dtype:
+            dtype = PintType.construct_from_quantity_string(scalars[0])
+        return cls._from_sequence([dtype.ureg.Quantity(x) for x in scalars])
+
     @classmethod
     def _from_factorized(cls, values, original):
         return cls(values, dtype=original.dtype)
@@ -510,6 +503,14 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
 
         return self._from_sequence(unique(self._data), dtype=self.dtype)
 
+    def __contains__(self, item) -> bool:
+        if not isinstance(item, _Quantity):
+            return False
+        elif pd.isna(item.magnitude):
+            return self.isna().any()
+        else:
+            return super().__contains__(item)
+
     @property
     def data(self):
         return self._data
@@ -584,7 +585,7 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
                 else:
                     return param
 
-            if isinstance(other, Series):
+            if isinstance(other, (Series, DataFrame)):
                 return NotImplemented
             lvalues = self.quantity
             other = validate_length(lvalues, other)
@@ -614,7 +615,7 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
 
             return res
 
-        op_name = ops._get_op_name(op, True)
+        op_name = f"__{op}__"
         return set_function_name(_binop, op_name, cls)
 
     @classmethod
@@ -632,12 +633,18 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
         return cls(quantity.magnitude, quantity.units)
 
     def __array__(self, dtype=None, copy=False):
-        if dtype in [None, "object"]:
+        if dtype is None or is_object_dtype(dtype):
             return self._to_array_of_quantity(copy=copy)
+        if (isinstance(dtype, str) and dtype == "string") or isinstance(
+            dtype, pd.StringDtype
+        ):
+            return pd.array([str(x) for x in self.quantity], dtype=pd.StringDtype())
+        if is_string_dtype(dtype):
+            return np.array([str(x) for x in self.quantity], dtype=str)
         return np.array(self._data, dtype=dtype, copy=copy)
 
     def _to_array_of_quantity(self, copy=False):
-        qtys = [item * self.units for item in self._data]
+        qtys = [self._Q(item, self._dtype.units) for item in self._data]
         return np.array(qtys, dtype="object", copy=copy)
 
     def searchsorted(self, value, side="left", sorter=None):
@@ -692,6 +699,49 @@ class PintArray(ExtensionArray, ExtensionOpsMixin):
             value = [item.to(self.units).magnitude for item in value]
         return arr.searchsorted(value, side=side, sorter=sorter)
 
+    def _reduce(self, name, skipna=True, **kwds):
+        """
+        Return a scalar result of performing the reduction operation.
+
+        Parameters
+        ----------
+        name : str
+            Name of the function, supported values are:
+            { any, all, min, max, sum, mean, median, prod,
+            std, var, sem, kurt, skew }.
+        skipna : bool, default True
+            If True, skip NaN values.
+        **kwargs
+            Additional keyword arguments passed to the reduction function.
+            Currently, `ddof` is the only supported kwarg.
+
+        Returns
+        -------
+        scalar
+
+        Raises
+        ------
+        TypeError : subclass does not define reductions
+        """
+        functions = {
+            "all": all,
+            "any": any,
+            "min": min,
+            "max": max,
+            "sum": sum,
+            "mean": np.mean,
+            "median": np.median,
+        }
+        if name not in functions:
+            raise TypeError(f"cannot perform {name} with type {self.dtype}")
+
+        if skipna:
+            quantity = self.dropna().quantity
+        else:
+            quantity = self.quantity
+
+        return functions[name](quantity)
+
 
 PintArray._add_arithmetic_ops()
 PintArray._add_comparison_ops()
@@ -869,4 +919,4 @@ def is_pint_type(obj):
         return False
 
 
-compat.upcast_types.append(PintArray)
\ No newline at end of file
+compat.upcast_types.append(PintArray)
